# -*- coding: utf-8 -*-

from menu import SubMenu
from RoadCentre import kernel, plugins, project
from PythonQt import QtGui
from gui import osmView
from concon import concon
import webbrowser

class CalcAreaMenu(SubMenu):
    def __init__(self):
        SubMenu.__init__(self, ('Участок расчёта', 'Участки расчёта'))
        a = self.menuMain.addAction('Создать расчёт')
        a.connect('triggered()', self.createCalculation)
        a = self.menuMain.addAction('Удалить')
        a.connect('triggered()', self.deleteArea)
        self.cancel_prog = self.menuMain.addAction('Отменить программы')
        self.cancel_prog.connect('triggered()', self.cancelPrograms)
        self.cancel_prog.enabled = False
        self.calc_objects = set()
        self.cursorOnMap = False
        a = self.menuMain.addAction('Открыть каталог')
        a.connect('triggered()', self.openFolder)
        

    def _selectedArea(self):
        for s in kernel.selection():
            if s.inherits('CalcArea'):
                yield s

    def createCalculation(self):
        for a in self._selectedArea():
            plugins.NetcalcPlugin.createCalculation(a)
            break
    
    def deleteArea(self):
        name_list = [ a.modelData(0,0) for a in self._selectedArea() ]
        if len(name_list) == 0:
            return
        res = QtGui.QMessageBox.question(0, "Вопрос", "Удалить выбранные участки расчёта?", QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
        if res != QtGui.QMessageBox.Yes:
            return
        for name in name_list:
            plugins.NetcalcPlugin.deleteArea(name)

    def menu(self, obj):
        self.cancel_prog.enabled = False
        self.cursorOnMap = osmView.rect.contains(osmView.mapFromGlobal(QtGui.QCursor.pos()))
        self.calc_objects = set()
        show_act = None
        m = SubMenu.menu(self, obj)
        for area in self._selectedArea():
            for obj_id in area.objects():
                tl = project.object(int(obj_id))
                if tl is not None and 50 <= tl.program <= 57:
                    self.calc_objects.add(tl.id)
                    self.cancel_prog.enabled = True
            if show_act is None and not self.cursorOnMap:
                show_act = m.addAction(u'Показать на карте')
                show_act.connect('triggered()', self.showOnMap)
        return m

    def cancelPrograms(self):
        for obj in self.calc_objects:
            plugins.JsonRpcClientPlugin.sendCommand('cnfStorage','app.putDprog2',{'obj_id':obj,'xml_data':'<Controller/>'})

    def showOnMap(self):
        for area in self._selectedArea():
            osmView.fitItem(area)
    
    def openFolder(self):
        for area in self._selectedArea():
            webbrowser.open(area.getFilePath(''))


class CalcObjectMenu(SubMenu):
    def __init__(self):
        SubMenu.__init__(self,('Расчёт','Расчёты'))
        a = self.menuMain.addAction('Загрузить в Авеню')
        a.connect('triggered()', self.uploadModel)
        a = self.menuMain.addAction('Запустить оптимизацию цикла')
        a.connect('triggered()', self.runCycleOptimization)
        a = self.menuMain.addAction('Запустить оптимизацию сдвигов')
        a.connect('triggered()', self.runOffsetOptimization)
        a = self.menuMain.addAction('Загрузить из Авеню')
        a.connect('triggered()', self.downloadModel)
        self.apply_to_rc = self.menuMain.addAction('Применить в ДК')
        self.apply_to_rc.connect('triggered()', self.applyToRC)
        self.apply_to_rc.enabled = False
        a = self.menuMain.addAction('Запустить эксперимент SUMO до оптимизации')
        a.connect('triggered()', self.runSumoExperimentBefore)
        self.run_sumo_after = self.menuMain.addAction('Запустить эксперимент SUMO после оптимизации')
        self.run_sumo_after.connect('triggered()', self.runSumoExperimentAfter)
        self.run_sumo_after.enabled = False
        
        a = self.menuMain.addAction('Изменить')
        a.connect('triggered()', self.changeCalculation)
        a = self.menuMain.addAction('Удалить')
        a.connect('triggered()', self.deleteCalculation)
        a = self.menuMain.addAction('Открыть каталог')
        a.connect('triggered()', self.openFolder)
    
    def _selectedCalculations(self):
        for s in kernel.selection():
            if s.inherits('CalculationObject'):
                yield s

    def menu(self, obj):
        self.apply_to_rc.enabled = False
        self.run_sumo_after.enabled = False
        for calc in self._selectedCalculations():
            if calc.completed:
                self.apply_to_rc.enabled = True
                self.run_sumo_after.enabled = True
                break
        return SubMenu.menu(self, obj)
    
    def uploadModel(self):
        for calc in self._selectedCalculations():
            plugins.NetcalcPlugin.runCalculation(calc,calc.optimizeNone)

    def downloadModel(self):
        for calc in self._selectedCalculations():
            plugins.NetcalcPlugin.runCalculation(calc,calc.downloadModel)

    def runCycleOptimization(self):
        for calc in self._selectedCalculations():
            plugins.NetcalcPlugin.runCalculation(calc,calc.optimizeCycle)

    def runOffsetOptimization(self):
        for calc in self._selectedCalculations():
            plugins.NetcalcPlugin.runCalculation(calc,calc.optimizeOffset)

    def applyToRC(self):
        for calc in self._selectedCalculations():
            plugins.NetcalcPlugin.applyCalculationToRc(calc)

    def runSumoExperimentBefore(self):
        for calc in self._selectedCalculations():
            plugins.NetcalcPlugin.runSumoExperiment(calc,False)

    def runSumoExperimentAfter(self):
        for calc in self._selectedCalculations():
            plugins.NetcalcPlugin.runSumoExperiment(calc,True)

    def changeCalculation(self):
        for calc in self._selectedCalculations():
            plugins.NetcalcPlugin.changeCalculation(calc)
            break

    def deleteCalculation(self):
        calc_list = [ a for a in self._selectedCalculations() ]
        if len(calc_list) == 0:
            return
        res = QtGui.QMessageBox.question(0, "Вопрос", "Удалить выбранные расчёты?", QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
        if res != QtGui.QMessageBox.Yes:
            return
        for calc in calc_list:
            plugins.NetcalcPlugin.deleteCalculation(calc)

    def openFolder(self):
        for calc in self._selectedCalculations():
            webbrowser.open(calc.getFilePath(''))

class DprogMenu(SubMenu):
    def __init__(self):
        SubMenu.__init__(self,('Динамическая программа','Динамические программы'))
        a = self.menuMain.addAction('Открыть в конфигураторе')
        a.connect('triggered()', self.openInConcon)
    
    def _selectedDprogs(self):
        for s in kernel.selection():
            if s.inherits('DynamicProgram'):
                yield s

    def openInConcon(self):
        cc = concon()
        for dprog in self._selectedDprogs():
            cc.start(["--edit-prog",dprog.fullFileName(),"--controller-config",dprog.config_name])
